/**
 * Copyright (C) 2003-2015, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF TOOLS to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * @file	fxpdftools.h
 * @brief	This header file includes all function in pdf tools.
 * @details	For application, just include this header file.
 *			It contains:<br>
 *			<ul>
 *			<li>1. Initialize Foxit PDFTools library and check license file. </li>
 *			<li>2. Convert image to PDF file.Image2PDF currently supports BMP, PNG, JPG, JPX, GIF,
 *					TIF (TIFF, including the images with single page or multiple pages) image formats
 *					and provides a great many options to control the properties of the output PDF files.  </li>
 *			<li>3. Convert office to PDF file.Office2PDF currently supports the popular Microsoft
 *					Office document formats, including doc, docx, xls, xlsx, ppt, pptx, and the converted
 *					PDF files can be compliant with PDF/A Standard. </li>
 *			<li>4. Add watermark to PDF file. PDFWatermark applies pre-made watermark, such as a text or
 *					an image, to PDF files. The watermark is saved as a configuration file with the extension
 *					.xml which was generated by Foxit PDFWatermark Configuration Tool (watermarkconfig.exe
 *					or watermarkconfig64.exe, a GUI application).</li>
 *			<li>5. Add header and footer to PDF file.PDFHeaderFooter applies pre-made header/footer to PDF
 *					files. The header/footer is saved as a configuration file with the extension .xml which
 *					was generated by Foxit PDFHeaderFooter Configuration Tool (headerfooterconfig.exe or
 *					headerfooterconfig64.exe depending on your operating system, a GUI application). </li>
 *			<li>6. Optimizer PDF file.  With the PDFOptimizer, users can optimize PDF files to enable them
 *					more suitable for electronic document exchange or space-saving document archiving.</li>
 *			<li>7. Compare PDF file. </li>
 *			<li>8. Destroy the Foxit PDFTools library. </li>
 *			</ul>
 *
 */

#ifndef _FXT_PDFTOOLS_H_
#define _FXT_PDFTOOLS_H_
#include <stdlib.h>


/** @brief Success. */
#define FXT_ERROR_SUCCESS		0
/** @brief File error: could not be opened. */
#define FXT_ERROR_FILE			-1
/** @brief format error: not a PDF or corrupted. */
#define FXT_ERROR_FORMAT		    -2
/** @brief Invalid password. */
#define FXT_ERROR_PASSWORD		-3
/** @brief Callback functions in handler are error. */
#define FXT_ERROR_HANDLER		-4
/** @brief This document is encrypted by digital certificate and current user doesn't have correct certificate. */ 
#define FXT_ERROR_CERT			-5
/** @brief Invalid param. */
#define FXT_ERROR_PARAM          -6
/** @brief Data cannot be found. */
#define FXT_ERROR_NOTFOUND       -7
/** @brief File is not permitted to compare.*/
#define FXT_ERROR_NOTPERMITTED   -8
/** @brief The Compare function cannot compare documents over 250 pages.*/
#define FXT_ERROR_OVERLIMIT      -9
/** @brief The Cannot create font.*/
#define FXT_ERROR_FONT           -10
/** @brief Invalid license.*/
#define FXT_ERROR_LICENSE        -11
/** @brief Failed to initialize GSDK Library.*/
#define FXT_ERROR_LIBRARY        -12
/** @brief Unsupported image format.*/
#define FXT_ERROR_IMAGEFORMAT	-13

/** @brief System error.*/
#define FXT_ERROR_SYSTEM			-14
/** @brief Crash error*/
#define FXT_ERROR_CRASH          -15
/** @brief Failed to save file.*/
#define FXT_ERROR_SAVE           -16
/** @brief Failed to open office app*/
#define FXT_ERROR_APP            -17
/** @brief Out of memory*/
#define FXT_ERROR_OOM            -18
/** @brief COM error*/
#define FXT_ERROR_COM            -19
/** @brief Other error */ 
#define FXT_ERROR_ERROR          -999


/** @brief Error information is from the running program.*/ 
#define CALLBACK_PDFTOOL_RUN_ERROR          0x0001
/** @brief Error information is from the command line params.*/
#define CALLBACK_PDFTOOL_PARAM_ERROR		0x0002
/** @brief Prompt information.*/
#define CALLBACK_PDFTOOL_MSG				0x0003
/** @brief Password is invalid.*/
#define CALLBACK_PDFTOOL_PASSWORD			0x0004

/**
* @brief		Declares a pointer type to the callback function for user's function method.
*
* @param[in]	userData	Pointer to user Data.
* @param[in]	mode		An integer value to specify the output mode of informations.
*							::CALLBACK_PDFTOOL_RUN_ERROR	Output information when the program is running failure.
*							::CALLBACK_PDFTOOL_PARAM_ERROR	Output information when the param is invalid.
*							::CALLBACK_PDFTOOL_MSG			Output the current state of program.
*							::CALLBACK_PDFTOOL_PASSWORD		Need input a password when opening a file, which is encrypted by a password.
* @param[in]	msg			Output information.
* @param[out]	isStop		If the value is true, stop the current application. Otherwise,continue running the application.
*
* @return		Return password when the mode is <b>CALLBACK_PDFTOOL_PASSWORD</b>, otherwise return NULL.
*/
typedef char*(*FXT_CallbackFun)(void* userData, int mode, char* msg, bool* isStop);

/**
* @brief		Initialize Foxit PDF SDK library, and check license.
*
* @param[in]	key			A path of license file.
*							If the path is empty, the application will find license file from current folder.
* @param[in]	screenFlag	Adjust whether output information to screen.
*							If the value is 0, don't output information to screen.Otherwise output information to screen.
*
* @return		::FXT_ERROR_SUCCESS	for success.
*				::FXT_ERROR_LIBRARY	if failed to initialize Foxit PDF SDK library.
*				::FXT_ERROR_LICENSE	the license is invalid.
*
* @attention	<b>Thread Unsafety</b>: this function is not thread safe.<br>
*				<b>The interface can be used only once in a program. </b>
*/

int FXT_InitLibrary(const wchar_t* key, int screenFlag);

/**
* @brief		Convert image to PDF by command line parameter.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to convert image to pdf.
*				::FXT_ERROR_ERROR		Failed to convert image to pdf.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to convert image to PDF by a string object.If user want to use 
*				command line to convert, example:"-xxxxx -i d:\test\input.jpg -o d:\result\output.pdf".
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int  FXT_Image2PDFRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);

/**
* @brief		Add watermark to PDF by command line parameter.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to add watermark for PDF file.
*				::FXT_ERROR_ERROR		Failed to add watermark for PDF file.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_FILE		Failed to parser configure file.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to add watermark to PDF by a string object.If user want to use command
*				line to add watermark, example:"-i d:\test\input.pdf -o d:\result\output.pdf -conf d:\test\configure.xml".
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int  FXT_WatermarkRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);

/**
* @brief		Optimizer PDF by command line parameter.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to optimizer PDF file.
*				::FXT_ERROR_ERROR		Failed to optimizer PDF file.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to optimizer PDF by a string object.If user want to use command
*				line to optimizer PDF, example:"-i d:\test\input.pdf -o d:\result\output.pdf".
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int  FXT_OptimizerRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);

#if defined(_MSC_VER)

/**
* @brief		Convert office to PDF by command line parameter.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to convert office  to PDF file.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to convert office to PDF by a string object.If user want to use command
*				line to convert office, example:"-i d:\test\input.doc -o d:\test\output.pdf".
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int  FXT_Office2PDFRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);

/**
* @brief		Compare two PDF files and output the result in a PDF file.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to compare PDF file.
*				::FXT_ERROR_ERROR		Failed to compare PDF file.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to compare PDF by a string object.If user want to use command
*				line to compare PDF, example:"d:\test\input_1.pdf d:\test\input_2.pdf d:\result\output.pdf"
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int FXT_CompareRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);

#endif  //_MSC_VER

/**
* @brief		Add header and footer to PDF by command line parameter.
*
* @param[in]	commandline	A string object to set the command line parameter.
* @param[in]	callback	Point to a callback function. For more detailed error and message reporting.
* @param[in]	userData	Pointer to user Data.
*
* @return		::FXT_ERROR_SUCCESS		Success to add header and footer for PDF file.
*				::FXT_ERROR_ERROR		Failed to add header and footer for PDF file.
*				::FXT_ERROR_LICENSE		License is Invalid.
*				::FXT_ERROR_PARAM		Parameter is error.
*				::FXT_ERROR_LIBRARY	    If failed to initialize Foxit PDF SDK library.
*
* @note			The interface supports to add header and footer to PDF by a string object.If user want to 
*				use command	line to add header and footer, example:"-i d:\test\input.pdf -o d:\result\output.pdf
*				-mode 1 -conf d:\test\configure.xml".
*
* @attention	<b>Thread Safety</b>: this function is thread safe.<br>
*/
int FXT_HeaderFooterRun(const char* commandline, FXT_CallbackFun callback, void* userData = 0);


/**
* @brief		Destroy the Foxit PDF SDK library.
*
* @attention	<b>Thread Unsafety</b>: this function is not thread safe.<br>
*				<b>The interface can be used only once in a program. </b>
*/

void FXT_DestroyLibrary();

#endif
